#! /bin/bash

# WebPy 0.1 for Mac and Linux by goobdoob - https://f95zone.com/members/goobdoob.334840/
# Based on WebPy 0.1 by Sam @ www.f95zone.com
# https://f95zone.com/threads/webpy-bat-v0-1-webp-based-image-compressor-rpa-extractor-included.12477/

# This script converts jpg and png files to webp format.

# Using:
# 
# cwebp 1.0.2 from https://developers.google.com/speed/webp/docs/precompiled
# rpatool from https://github.com/Shizmob/rpatool

# cwebp binary is included for MacOS. For Linux, install the relevant package.

# Important - this modifies images in place. Do not run it on your only copy of the images!

# Usage:
# 
# You can run this script by passing in the directory to work on, or type in the
# directory when prompted. On a Mac, you can drag the .app file to the terminal window
# when prompted for the game directory.
#
# If you're working on a Ren'Py game, you can choose to extract and optionally delete
# RPA files.
#
# After that, input the quality you want and how many threads to use.

# Changelog:
# v0.1:
# Initial Mac/Linux version, based on WebPy v0.1 by Sam.

ARG1=$1
num_args=$#

version="0.1"

clear
#Splash Screen
function init
{
    echo "WebPy for Mac and Linux v$version"
    echo "by goobdoob @ www.f95zone.com"
    echo "based on the original version by Sam @ www.f95zone.com"
    echo
    rpatool=`dirname "$0"`"/rpatool"
    if ! [ -e "$rpatool" ]; then
        echo "Error - cannot find rpatool in the directory with this script. Exiting"
        exit
    fi
    unameout=$("uname")
    if [ $unameout = "Darwin" ]; then
        cwebp=`dirname $0`"/cwebp"
        if ! [ -e "$cwebp" ]; then
            echo "Error - cannot find cwebp in the directory with this script. Exiting"
            exit
        fi
    else
        cwebp="cwebp"

        if [ -x "$(command -v cwepb)" ]; then
            echo "Error - cannot find cwebp. Please install the correct package. Exiting"
            exit
        fi
    fi
    
    if [ $num_args -ne 1 ]; then
        echo "Please drag + drop the directory with the images you are want to convert"
        echo " into the terminal, or type in the path to the game."
        read convdir
    else
        echo "Working with directory $ARG1"
        convdir=$ARG1
    fi
    cd "$convdir"
    pwd
    
    echo
    echo  " *** This tool converts png and jpg images to WebP format        ***"
    echo  " *** If you select a quality setting other than \"lossless\"     ***"
    echo  " *** this conversion can NOT be reversed.                        ***"
    echo  " *** Make sure you have a backup of any files before continuing. ***"
    echo  " *** Use at your own risk.                                       ***"
    echo
    rpa_choice
    do_convert
}

function rpa_choice
{
    echo "Do you wish to extract RPA files? [Y/n/q]"
    echo
    read choice_rpa
    if [ -z "$choice_rpa" ]; then
        choice_rpa="Y"
    fi
    case "$choice_rpa" in
    "y"|"Y")
        extract_rpas
	    echo  " Finished extracting RPAs"
	    echo
	    del_rpa_choice
        ;;
    "n"|"N")
        echo "Not extracting RPAs"
        ;;
    "q")
        exit
        ;;
    *)
        echo "Please make a valid selection"
        rpa_choice
        ;;
    esac
}

function extract_rpas
{
    # if we're extracting RPAs, then let's assume we're in a Ren'Py game
    if [ -d Contents/Resources/autorun/game ]; then
        cd Contents/Resources/autorun/game
    elif [ -d game ]; then
        cd game
    fi
    echo  "----------------------------------------------------"
	echo  " Extracting RPAs from directory "`pwd`
    for f in *.rpa
    do
        python "$rpatool" -x "$f"
    done
    # The find below puts the files extracted from the RPAs in the current directory.
    # This is possibly wrong, so use the for loop above instead.
    #find . -name \*.rpa -print0 | xargs -0 -I % python "$rpatool" -x % -o %
}

function del_rpa_choice
{
    echo "Do you wish to delete RPA files? [y/N/q]"
    echo
    read choice_del_rpa
    if [ -z "$choice_del_rpa" ]; then
        choice_rpa="N"
    fi
    case "$choice_rpa" in
    "y"|"Y")
	    echo  "----------------------------------------------------"
	    echo  " Deleting RPAs from directory `pwd`"
        rm *.rpa
	    echo  " Finished deleting RPAs"
	    echo
        ;;
    "n"|"N")
        echo "Not deleting RPAs"
        ;;
    "q")
        exit
        ;;
    *)
        echo "Please make a valid selection"
        del_rpa_choice
        ;;
    esac
}

function do_convert
{
    # bogus value to use in loop test
    quality=1000
    quality_flag="-lossless"
    while [ $quality -eq 1000 ]
    do
        echo
        echo "What quality should the converted WebP images be?"
        echo "   Input a number from 0-100, or anything else for lossless"
        read choice_quality
        if [ -n "$choice_quality" ] && [ "$choice_quality" -eq "$choice_quality" ]; then
            quality=$choice_quality
            if [ $quality -lt 0 ]; then
                quality=0
            elif [ $quality -gt 100 ]; then
                quality=100
            fi
            echo
            echo "Using quality $quality"
            my_quality=$quality
            quality_flag="-q $quality"
        else
            echo
            echo "Using lossless quality"
            my_quality="lossless"
            # unused value to get out of loop
            quality=100
        fi
    done
    
    threads=0
    while [ $threads -eq 0 ]
    do
        echo
        echo "How many threads do you want to use? [4]"
        read choice_threads
        if [ -n "$choice_threads" ] && [ "$choice_threads" -eq "$choice_threads" ]; then
            threads=$choice_threads
            # limit to something logical
            if [ $threads -gt 16 ]; then
                threads=16
            fi
        elif [ -n "$choice_threads" ]; then
            echo "Invalid number of threads specified. Please try again."
        else
            threads=4
        fi
    done
    
	echo  "----------------------------------------------------"
    echo  "Converting jpg and png files to webp using $threads threads and $my_quality quality"    
    find . -name \*.jpg -print0 | xargs -0 -P $threads -I % "$cwebp" $qualityflag % -o % -quiet
    find . -name \*.png -print0 | xargs -0 -P $threads -I % "$cwebp" $qualityflag % -o % -quiet
    echo "Conversion done!"
}


init